@@ -182,7 +182,7 @@ impl YMap {
         entry.ok_or_else(|| PyKeyError::new_err(format!("{key}")))
     }
 
-    /// Returns an iterator that can be used to traverse over all entries stored within this
+    /// Returns an item view that can be used to traverse over all entries stored within this
     /// instance of `YMap`. Order of entry is not specified.
     ///
     /// Example:
@@ -199,23 +199,23 @@ impl YMap {
     ///     for (key, value) in map.entries(txn)):
     ///         print(key, value)
     /// ```
-    pub fn items(&self) -> YMapIterator {
-        match &self.0 {
-            SharedType::Integrated(val) => unsafe {
-                let this: *const Map = val;
-                let shared_iter = InnerYMapIterator::Integrated((*this).iter());
-                YMapIterator(ManuallyDrop::new(shared_iter))
-            },
-            SharedType::Prelim(val) => unsafe {
-                let this: *const HashMap<String, PyObject> = val;
-                let shared_iter = InnerYMapIterator::Prelim((*this).iter());
-                YMapIterator(ManuallyDrop::new(shared_iter))
-            },
-        }
+    pub fn items(&self) -> ItemView {
+        ItemView(&self.0)
     }
 
-    pub fn __iter__(&self) -> YMapKeyIterator {
-        YMapKeyIterator(self.items())
+    pub fn keys(&self) -> KeyView {
+        let inner: *const _ = &self.0;
+        KeyView(inner)
+    }
+
+    pub fn __iter__(&self) -> KeyIterator {
+        let inner: *const _ = &self.0;
+        KeyIterator(YMapIterator::from(inner))
+    }
+
+    pub fn values(&self) -> ValueView {
+        let inner: *const _ = &self.0;
+        ValueView(inner)
     }
 
     pub fn observe(&mut self, f: PyObject) -> PyResult<ShallowSubscription> {
@@ -267,6 +267,131 @@ impl YMap {
     }
 }
 
+#[pyclass(unsendable)]
+pub struct ItemView(*const SharedType<Map, HashMap<String, PyObject>>);
+
+#[pymethods]
+impl ItemView {
+    fn __iter__(slf: PyRef<Self>) -> YMapIterator {
+        YMapIterator::from(slf.0)
+    }
+
+    fn __len__(&self) -> usize {
+        unsafe {
+            match &*self.0 {
+                SharedType::Integrated(map) => map.len() as usize,
+                SharedType::Prelim(map) => map.len(),
+            }
+        }
+    }
+
+    fn __str__(&self) -> String {
+        let vals: String = YMapIterator::from(self.0)
+            .map(|(key, val)| format!("({key}, {val})"))
+            .collect::<Vec<String>>()
+            .join(", ");
+        format!("{{{vals}}}")
+    }
+
+    fn __repr__(&self) -> String {
+        let data = self.__str__();
+        format!("ItemView({data})")
+    }
+
+    fn __contains__(&self, el: PyObject) -> bool {
+        let kv: Result<(String, PyObject), _> = Python::with_gil(|py| el.extract(py));
+        kv.ok()
+            .and_then(|(key, value)| unsafe {
+                match &*self.0 {
+                    SharedType::Integrated(map) if map.contains(&key) => map.get(&key).map(|v| {
+                        Python::with_gil(|py| v.into_py(py).as_ref(py).eq(value)).unwrap_or(false)
+                    }),
+                    SharedType::Prelim(map) if map.contains_key(&key) => map
+                        .get(&key)
+                        .map(|v| Python::with_gil(|py| v.as_ref(py).eq(value).unwrap_or(false))),
+                    _ => None,
+                }
+            })
+            .unwrap_or(false)
+    }
+}
+
+#[pyclass(unsendable)]
+pub struct KeyView(*const SharedType<Map, HashMap<String, PyObject>>);
+
+#[pymethods]
+impl KeyView {
+    fn __iter__(slf: PyRef<Self>) -> KeyIterator {
+        KeyIterator(YMapIterator::from(slf.0))
+    }
+
+    fn __len__(&self) -> usize {
+        unsafe {
+            match &*self.0 {
+                SharedType::Integrated(map) => map.len() as usize,
+                SharedType::Prelim(map) => map.len(),
+            }
+        }
+    }
+
+    fn __str__(&self) -> String {
+        let vals: String = YMapIterator::from(self.0)
+            .map(|(key, _)| key)
+            .collect::<Vec<String>>()
+            .join(", ");
+        format!("{{{vals}}}")
+    }
+
+    fn __repr__(&self) -> String {
+        let data = self.__str__();
+        format!("KeyView({data})")
+    }
+
+    fn __contains__(&self, el: PyObject) -> bool {
+        let key: Result<String, _> = Python::with_gil(|py| el.extract(py));
+        key.ok()
+            .map(|key| unsafe {
+                match &*self.0 {
+                    SharedType::Integrated(map) => map.contains(&key),
+                    SharedType::Prelim(map) => map.contains_key(&key),
+                }
+            })
+            .unwrap_or(false)
+    }
+}
+
+#[pyclass(unsendable)]
+pub struct ValueView(*const SharedType<Map, HashMap<String, PyObject>>);
+
+#[pymethods]
+impl ValueView {
+    fn __iter__(slf: PyRef<Self>) -> ValueIterator {
+        ValueIterator(YMapIterator::from(slf.0))
+    }
+
+    fn __len__(&self) -> usize {
+        unsafe {
+            match &*self.0 {
+                SharedType::Integrated(map) => map.len() as usize,
+                SharedType::Prelim(map) => map.len(),
+            }
+        }
+    }
+
+    fn __str__(&self) -> String {
+        let vals: String = YMapIterator::from(self.0)
+            .map(|(_, v)| v.to_string())
+            .collect::<Vec<String>>()
+            .join(", ");
+        format!("{{{vals}}}")
+    }
+
+    fn __repr__(&self) -> String {
+        let data = self.__str__();
+        format!("ValueView({data})")
+    }
+}
+
 pub enum InnerYMapIterator {
     Integrated(MapIter<'static>),
     Prelim(std::collections::hash_map::Iter<'static, String, PyObject>),
@@ -281,6 +406,25 @@ impl Drop for YMapIterator {
     }
 }
 
+impl From<*const SharedType<Map, HashMap<String, PyObject>>> for YMapIterator {
+    fn from(inner_map_ptr: *const SharedType<Map, HashMap<String, PyObject>>) -> Self {
+        unsafe {
+            match &*inner_map_ptr {
+                SharedType::Integrated(val) => {
+                    let this: *const Map = val;
+                    let shared_iter = InnerYMapIterator::Integrated((*this).iter());
+                    YMapIterator(ManuallyDrop::new(shared_iter))
+                }
+                SharedType::Prelim(val) => {
+                    let this: *const HashMap<String, PyObject> = val;
+                    let shared_iter = InnerYMapIterator::Prelim((*this).iter());
+                    YMapIterator(ManuallyDrop::new(shared_iter))
+                }
+            }
+        }
+    }
+}
+
 impl Iterator for YMapIterator {
     type Item = (String, PyObject);
 
@@ -305,10 +449,10 @@ impl YMapIterator {
 }
 
 #[pyclass(unsendable)]
-pub struct YMapKeyIterator(YMapIterator);
+pub struct KeyIterator(YMapIterator);
 
 #[pymethods]
-impl YMapKeyIterator {
+impl KeyIterator {
     fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
         slf
     }
@@ -317,6 +461,19 @@ impl YMapKeyIterator {
     }
 }
 
+#[pyclass(unsendable)]
+pub struct ValueIterator(YMapIterator);
+
+#[pymethods]
+impl ValueIterator {
+    fn __iter__(slf: PyRef<Self>) -> PyRef<Self> {
+        slf
+    }
+    fn __next__(mut slf: PyRefMut<Self>) -> Option<PyObject> {
+        slf.0.next().map(|(_, v)| v)
+    }
+}
+
 /// Event generated by `YMap.observe` method. Emitted during transaction commit phase.
 #[pyclass(unsendable)]
 pub struct YMapEvent {
@@ -362,6 +519,13 @@ impl YMapEvent {
         }
     }
 
+    pub fn __repr__(&mut self) -> String {
+        let target = self.target();
+        let keys = self.keys();
+        let path = self.path();
+        format!("YMapEvent(target={target}, keys={keys}, path={path})")
+    }
+
     /// Returns an array of keys and indexes creating a path from root type down to current instance
     /// of shared type (accessible via `target` getter).
     pub fn path(&self) -> PyObject {
